/* @alwatr/async-queue v1.3.5 */

// src/main.ts
import { newFlatomise } from "@alwatr/flatomise";
import { packageTracer } from "@alwatr/package-tracer";
__dev_mode__: packageTracer.add("@alwatr/async-queue", "1.3.5");
var AsyncQueue = class {
  constructor() {
    /**
     * A record of task IDs and their corresponding last queued task promises.
     */
    this.queue__ = {};
  }
  /**
   * Push a async task to the queue.
   *
   * @param taskId task id
   * @param task async task
   * @returns A promise that resolves when the task is done.
   *
   * @example
   * ```typescript
   * const queue = new AsyncQueue();
   *
   * function longTask() {
   *  queue.push('longTaskId', async () => {
   *   // ...
   * });
   * ```
   */
  async push(taskId, task) {
    const flatomise = newFlatomise();
    const previousTaskPromise = this.queue__[taskId];
    this.queue__[taskId] = flatomise.promise;
    try {
      await previousTaskPromise;
    } catch (_e) {
    }
    setTimeout(() => {
      task().then(flatomise.resolve, flatomise.reject).then(() => {
        if (this.queue__[taskId] === flatomise.promise) {
          delete this.queue__[taskId];
        }
      });
    }, 0);
    return flatomise.promise;
  }
  /**
   * Check if the task running in the queue.
   *
   * @param taskId task id
   * @returns true if the task is running, otherwise false.
   * @example
   * ```typescript
   * if (queue.isRunning('longTaskId')) {
   *  // ...
   * }
   * ```
   */
  isRunning(taskId) {
    return this.queue__[taskId] !== void 0;
  }
  /**
   * Wait for the all tasks in the queue to finish.
   *
   * @param taskId task id
   * @returns A promise that resolves when all tasks are done.
   * @example
   * ```typescript
   * await queue.waitForFinish('longTaskId');
   * ```
   */
  waitForFinish(taskId) {
    return this.queue__[taskId] ?? Promise.resolve();
  }
  /**
   * Wait for the all tasks in the queue to finish.
   * @returns A promise that resolves when all tasks are done.
   * @example
   * ```typescript
   * await queue.waitForAllFinish();
   * ```
   */
  waitForAllFinish() {
    return Promise.all(Object.values(this.queue__));
  }
};
export {
  AsyncQueue
};
/*! For license information please see main.mjs.LEGAL.txt */
//# sourceMappingURL=main.mjs.map
